@@ -51,9 +51,11 @@
 #include "global.h"
 #include "netlist.h"
 
-
 #define MLFIPRIV(ctx)        ((struct mlfiPriv *) smfi_getpriv(ctx))
 
+/* Initialize as Encryption Mode */
+int ismode_delete = 0;
+
 /*
  *  sfsistat mlfi_connect
  */
@@ -128,15 +130,22 @@ mlfi_connect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *addr)
     priv->mlfi_rcptcheck = BEFOR_RCPTCHECK;
 
     /* set encription flag */
-    priv->mlfi_encstatus = ENC;
+    priv->mlfi_encstatus = ENC; //initialization for Encryption mode and Delete mode.
+
+    /* Only for Encription mode --- from here --- */
+    if (!ismode_delete) {
+    
+        /* set AutoBcc check flag */
+        if (priv->mlfi_conf->cf_autobccoption[0] == 'Y' || priv->mlfi_conf->cf_autobccoption[0] == 'y') {
+	    priv->mlfi_bccoption = ENABLE_AUTOBCC;
+        }
+        else {
+	    priv->mlfi_bccoption = DISABLE_AUTOBCC;
+        }
 
-    /* set AutoBcc check flag */
-    if (priv->mlfi_conf->cf_autobccoption[0] == 'Y' || priv->mlfi_conf->cf_autobccoption[0] == 'y') {
-	priv->mlfi_bccoption = ENABLE_AUTOBCC;
-    }
-    else {
-	priv->mlfi_bccoption = DISABLE_AUTOBCC;
     }
+    /* Only for Encription mode --- to here --- */
+
 
     /* create mailinfo structure */
     minfo = malloc(sizeof *minfo);
@@ -195,6 +204,7 @@ mlfi_envfrom(SMFICTX *ctx, char **envfrom)
         return SMFIS_TEMPFAIL;
     }
 
+
     DEBUGLOG("mlfi_envfrom start");
 
     /* get mail addr */
@@ -205,7 +215,15 @@ mlfi_envfrom(SMFICTX *ctx, char **envfrom)
         return SMFIS_TEMPFAIL;
     }
 
-    /* domain null check */
+    if (ismode_delete) {
+        /* All sender accepted when SaMMA is running in delete mode. */
+        /* save from addr temporally */
+        priv->mlfi_savefrom = fromaddr;
+        DEBUGLOG("Sender Accepted");
+        return SMFIS_CONTINUE;
+    }
+
+     /* domain null check */
     fqdn = strchr(fromaddr, ATMARK);
     if (fqdn == NULL) {
         free(fromaddr);
@@ -264,6 +282,8 @@ mlfi_envrcpt(SMFICTX *ctx, char **rcptto)
         return SMFIS_TEMPFAIL;
     }
 
+    DEBUGLOG("get mail addr: %s", rcptaddr);
+
     /* push rcpt addr */
     if (push_rcptlist(&(priv->mlfi_savercpt), rcptaddr) != 0) {
 	mlfi_cleanup(ctx, MLFIABORT);
@@ -298,59 +318,82 @@ mlfi_header(SMFICTX *ctx, char *headerf, char *headerv)
 
     DEBUGLOG("mlfi_header start");
 
+    // Cases for priv->mlfi_encstatus == NOT_ENC:
+    //  o Not-Encryption due to the fromDB.
+    //  o Not-Encryption as a result of the following rcptcheck.
     if (priv->mlfi_encstatus == NOT_ENC) {
-	return SMFIS_CONTINUE;
+        return SMFIS_CONTINUE;
     }
 
-    /* START ADD: 20150323*/
+    /* START ADD: 20150323 */
     /* check loop encryption */
     if ((strcmp(priv->mlfi_conf->cf_loopcheck, LOOP_CHECK) == 0 ) &&
-          (strcmp(headerf, XHEADER) == 0) && 
-          (strcmp(headerv, XHEADER_YES) == 0)) {
-	mlfi_cleanup(ctx, MLFIABORT);
+	(strcmp(headerf, XHEADER) == 0) && 
+	(strcmp(headerv, XHEADER_YES) == 0)) {
+        mlfi_cleanup(ctx, MLFIABORT);
         return SMFIS_ACCEPT;
     }
     /* END ADD: 20150323*/
 
     /* RcptTo Check */
     if (priv->mlfi_rcptcheck == BEFOR_RCPTCHECK) {
-	if (priv->mlfi_bccoption == ENABLE_AUTOBCC) {
-	    ret = add_bccaddr(priv->mlfi_conf, priv->mlfi_savefrom,
-			     &priv->mlfi_savercpt, &priv->mlfi_savebcc);
-	    if (ret != SUCCESS) {
-		mlfi_cleanup(ctx, MLFIABORT);
-		return SMFIS_TEMPFAIL;
-	    }
-	}
 
-	ret = search_rcptaddr(priv->mlfi_conf, priv->mlfi_savefrom, 
-			      priv->mlfi_savercpt, &passlist, &rdmpasslist);
-	if (ret < 0) {
-	    free_rcptinfo(passlist);
-	    free_rcptinfo(rdmpasslist);
-	    mlfi_cleanup(ctx, MLFIABORT);
-	    return SMFIS_TEMPFAIL;
-	}
-	if (ret == NOT_ENC) {
-	    free_rcptinfo(passlist);
-	    free_rcptinfo(rdmpasslist);
-	    if (priv->mlfi_bccoption == DISABLE_AUTOBCC) {
-		mlfi_cleanup(ctx, MLFIABORT);
-		return SMFIS_ACCEPT;
-	    }
-	    /* save envription status */
-	    priv->mlfi_encstatus = NOT_ENC;
-	    priv->mlfi_rcptcheck = AFTER_RCPTCHECK;
-	    return SMFIS_CONTINUE;
-	}
+        if (!ismode_delete) {
+            /* Encryption Mode */
+            if (priv->mlfi_bccoption == ENABLE_AUTOBCC) {
+                ret = add_bccaddr(priv->mlfi_conf, priv->mlfi_savefrom,
+                        &priv->mlfi_savercpt, &priv->mlfi_savebcc);
+                if (ret != SUCCESS) {
+                    mlfi_cleanup(ctx, MLFIABORT);
+                    return SMFIS_TEMPFAIL;
+                }
+            }
 
-	/* save encryption address list */
-	priv->mlfi_passlist = passlist;
-	priv->mlfi_rdmpasslist = rdmpasslist;
+            ret = search_rcptaddr(priv->mlfi_conf, priv->mlfi_savefrom, 
+                    priv->mlfi_savercpt, &passlist, &rdmpasslist);
+        } else {
+            /* Delete Mode */
+            ret = search_rcptaddr_delmode(priv->mlfi_conf, 
+                    priv->mlfi_savercpt, &passlist, &rdmpasslist);
+            // Delete recipients are added to passlist.
+        }
 
-	priv->mlfi_rcptcheck = AFTER_RCPTCHECK;
+        if (ret < 0) {
+            free_rcptinfo(passlist);
+            free_rcptinfo(rdmpasslist);
+            mlfi_cleanup(ctx, MLFIABORT);
+            return SMFIS_TEMPFAIL;
+        }
+        if (ret == NOT_ENC) {
+            free_rcptinfo(passlist);
+            free_rcptinfo(rdmpasslist);
+            if (ismode_delete) {
+                mlfi_cleanup(ctx, MLFIABORT);
+                return SMFIS_ACCEPT;
+            }
+            if (priv->mlfi_bccoption == DISABLE_AUTOBCC) {
+                mlfi_cleanup(ctx, MLFIABORT);
+                return SMFIS_ACCEPT;
+            }
+            /* save encription status */
+            priv->mlfi_encstatus = NOT_ENC;
+            priv->mlfi_rcptcheck = AFTER_RCPTCHECK;
+            return SMFIS_CONTINUE;
+        }
+
+        /* save encryption address list */
+        priv->mlfi_passlist = passlist;
+        priv->mlfi_rdmpasslist = rdmpasslist;
+
+        priv->mlfi_rcptcheck = AFTER_RCPTCHECK;
     }
 
+
+#ifdef DEBUG
+listvardump(passlist);
+#endif
+//DEBUGLOG("headerf= %s", headerf);
+//DEBUGLOG("headerv= %s", headerv);
     /* create header line */
     len = strlen(headerf) + strlen(headerv) + 4;
     header = malloc(len + 1);
@@ -469,10 +512,17 @@ mlfi_eom(SMFICTX *ctx)
 	return SMFIS_ACCEPT;
     }
 
-    /* Encryption Process */
-    ret = zip_convert_mail(ctx, priv->mlfi_minfo, priv->mlfi_conf, 
-			   priv->mlfi_savefrom, priv->mlfi_passlist, 
-		 	   priv->mlfi_rdmpasslist);
+    if (ismode_delete) {
+        /* Delete Attachments */
+        ret = delete_attachments_mail(ctx, priv->mlfi_minfo, 
+                priv->mlfi_conf, priv->mlfi_savefrom,
+                priv->mlfi_rdmpasslist);
+    } else {
+        /* Encryption Process */
+        ret = zip_convert_mail(ctx, priv->mlfi_minfo, priv->mlfi_conf, 
+                priv->mlfi_savefrom, priv->mlfi_passlist, 
+                priv->mlfi_rdmpasslist);
+    }
 
     /* Reset MailSave Information */
     mailsave_reset(priv->mlfi_minfo);
@@ -604,14 +654,14 @@ call_command(void *arg)
 void
 usage(char *arg)
 {
-    fprintf(stderr, "usage: %s [-t timeout] [config file]\n", arg);
+    fprintf(stderr, "usage: %s [-d] [-t timeout] [config file]\n", arg);
 }
 
 int
 main(int argc, char *argv[])
 {
 
-    char  *args = "t:";
+    char  *args = "dt:";
     char  *confname;
     char  *tmp;
     char   c;
@@ -652,6 +702,11 @@ main(int argc, char *argv[])
 
             timeout = (int) tmp_val;
             break;
+        case 'd':
+            /* Delete Mode */
+            ismode_delete = 1;
+
+            break;
 
         /* error */
         default:
@@ -676,6 +731,9 @@ main(int argc, char *argv[])
 	exit(1);
     }
 
+    /* Select config entries. */
+    select_cfentry();
+
     /* read config file */
     if (reload_config(confname) != 0) {
 	fprintf(stderr, STDERR_CONFIG_READ, confname);
@@ -744,3 +802,23 @@ main(int argc, char *argv[])
 
     exit(0);
 }
+
+/* for debug  */
+#ifdef DEBUG
+void 
+listvardump(struct rcptinfo *passlist)
+{
+    int i;
+    struct rcptinfo *x = NULL;
+
+    for (x = passlist; x != NULL; x = x->Next) {
+        for (i = 0; (x->rcptlist + i)->rcpt_addr != NULL; i++) {
+            DEBUGLOG("p= %p, addr= %s, p->Next= %p",
+                     x, (x->rcptlist + i)->rcpt_addr, x->Next);
+        }
+    }
+
+    return;
+}
+#endif
+
