@@ -117,6 +117,96 @@ mailzip_clean(struct mailzip mz)
         free(mz.attachfilename);
     }
 }
+/*
+ * delete_attachmetns_mail
+ *
+ * Functions:
+ *     - Parse mail and get a list of the names of the all files 
+ *       attached on the original mail, then delete the files.
+ *     - Make a path string of the DeleteListName file.
+ *     - Creat and write the DeleteListName file.
+ *     - Attach the DeleteListName file to the copied mail.
+ *     - Send the copied mail to the rcipients who are the target
+ *       of the attachments-deletion.
+ *     - Unlink the DeleteListName file.
+ *     - Delete recipients of the original mail who are 
+ *       the target of the attachments-deletion.
+ * Args:
+ *     See blow.
+ * Returns:
+ *     ZIP_CONVERT_ACCEPT
+ *     ZIP_CONVERT_FAILED
+ *     ZIP_CONVERT_INVALID_FILENAME
+ *     ZIP_CONVERT_SUCCESS
+ */
+extern int
+delete_attachments_mail(SMFICTX *ctx, struct mailinfo *minfo, 
+                 struct config *cfg, char *from, struct rcptinfo *rdmpasslist)
+{
+    struct mailzip mz;
+    struct rcptinfo *p;
+    int i, ret;
+    char *temp = NULL, *rcptaddr;
+
+    memset(&mz, 0, sizeof(struct mailzip));
+
+    /* parse mail */
+    ret = parse_mail(minfo, cfg, &mz);
+    if ((ret == PM_NO_MULTIPART) || (ret == PM_NO_ATTACHFILE)) {
+	mailzip_clean(mz);
+	return ZIP_CONVERT_ACCEPT;
+    } else if (ret == PM_FAILED) {
+	mailzip_clean(mz);
+	return ZIP_CONVERT_FAILED;
+    } else if (ret == PM_INVALID_FILENAME) {
+	mailzip_clean(mz);
+	return ZIP_CONVERT_INVALID_FILENAME;
+    }
+
+    /* Make a path of the DeleteListName file */
+    if (mk_encpath(cfg, &mz) == -1) {
+	mailzip_clean(mz);
+	return ZIP_CONVERT_FAILED;
+    }
+
+    if (rdmpasslist != NULL) {
+	/* deletion */
+	if (mk_deletelistfile(cfg, &mz) != 0) {
+	    mailzip_clean(mz);
+	    return ZIP_CONVERT_FAILED;
+	}
+
+	/* add attach file */
+	if (add_attach_file(&mz, rdmpasslist) != 0) {
+	    mailzip_clean(mz);
+	    return ZIP_CONVERT_FAILED;
+	}
+    	/* send mail */
+	if (sendmail(&mz, cfg, rdmpasslist, from) != 0) {
+	    mailzip_clean(mz);
+	    return ZIP_CONVERT_FAILED;
+        }
+        /* unlink the temporary file */
+        if (unlink(mz.encfilepath) == -1) {
+            log(ERR_FILE_REMOVE, "delete_attachments_mail", unlink(mz.encfilepath));
+            mailzip_clean(mz);
+            return ZIP_CONVERT_FAILED;
+        }
+
+        for (i = 0; (rdmpasslist->rcptlist + i)->rcpt_addr != NULL; i++) {
+            rcptaddr = (rdmpasslist->rcptlist + i)->rcpt_addr;
+            /* delete from envelope to */
+            if (smfi_delrcpt(ctx, rcptaddr) == MI_FAILURE) {
+                log(ERR_DELETE_ADDRESS, "delete_attachment_mail", rcptaddr);
+	        mailzip_clean(mz);
+                return ZIP_CONVERT_FAILED;
+            }
+        }
+    } 
+
+    mailzip_clean(mz);
+    return ZIP_CONVERT_SUCCESS;
+}
 
 extern int
 zip_convert_mail(SMFICTX *ctx, struct mailinfo *minfo, struct config *cfg,
@@ -318,7 +408,6 @@ add_attach_file(struct mailzip *mz, struct rcptinfo *rcpt)
     int index, i;
     const char *filename;
 
-
     pthread_mutex_lock(&gmime_lock);
     object = g_mime_message_get_mime_part(mz->message);
 
@@ -671,7 +760,9 @@ parse_mail(struct mailinfo *minfo, struct config *cfg, struct mailzip *mz)
     char *tmpdir, *tmpstr;
     int rfd, len;
     int ret;
+    char *cfg_encdir = NULL;
 
+    cfg_encdir = ismode_delete ? cfg->cf_tmpdir : cfg->cf_encryptiontmpdir;
     pthread_mutex_lock(&gmime_lock);
 
     /* create object */
@@ -758,7 +849,7 @@ parse_mail(struct mailinfo *minfo, struct config *cfg, struct mailzip *mz)
     }
 
     /* make zip directory */
-    len = strlen(cfg->cf_encryptiontmpdir) + strlen(TMPDIR_TMPL) + 2;
+    len = strlen(cfg_encdir) + strlen(TMPDIR_TMPL) + 2;
     tmpstr = (char *)malloc(len);
 
     if (tmpstr == NULL) {
@@ -769,7 +860,7 @@ parse_mail(struct mailinfo *minfo, struct config *cfg, struct mailzip *mz)
 	pthread_mutex_unlock(&gmime_lock);
 	return(PM_FAILED);
     }
-    snprintf(tmpstr, len, FILE_PATH, cfg->cf_encryptiontmpdir, TMPDIR_TMPL);
+    snprintf(tmpstr, len, FILE_PATH, cfg_encdir, TMPDIR_TMPL);
 
     tmpdir = mkdtemp(tmpstr);
     if (tmpdir == NULL) {
@@ -800,7 +891,6 @@ parse_mail(struct mailinfo *minfo, struct config *cfg, struct mailzip *mz)
 	pthread_mutex_unlock(&gmime_lock);
         if (ret == -2) {
 /* if cannot convert character code then set filename is unknown file */
-printf("errror \n");
             /* could not detect/convert attached filename's encoding
              * returning specific value */
             return(PM_INVALID_FILENAME);
@@ -846,8 +936,7 @@ drop_file(GMimeObject *object, struct config *cfg, struct mailzip *mz)
     index = g_mime_multipart_get_count((GMimeMultipart *)object);
 #endif
 
-
-    for (i = index - 1; i > 0; i--) {
+    for (i = index - 1; i >= 0; i--) {
 	att = g_mime_multipart_get_part((GMimeMultipart *)object, i);
 
 	if (GMIME_IS_MESSAGE_PART(att)) {
@@ -865,11 +954,13 @@ drop_file(GMimeObject *object, struct config *cfg, struct mailzip *mz)
 	    continue;
 
 	} else {
+DEBUGLOG("getting filename...");
 
 	    /* get filename */
 	    filename = g_mime_part_get_filename((GMimePart *)att);
 
 	    if (filename == NULL) {
+DEBUGLOG("Failed to get filename when index = %d", i);
 #ifndef GMIME24 
 	        g_object_unref (att);
 #endif
@@ -897,7 +988,9 @@ drop_file(GMimeObject *object, struct config *cfg, struct mailzip *mz)
 	/* convert str code */
 	if (dg_str2code(tmpname, &tmpfilename, STR_UTF8, cfg->cf_strcode) != 0) {
             /* START ADD 20150323 */
-            /* if convert str code error then  make unknown file*/
+            /* If the conversion of the str2code is failed, the name of 
+             * the attaching file is set by cf_attachmentfilealias.
+             */
             tmpfilename = strdup(cfg->cf_attachmentfilealias);
             if (tmpfilename == NULL) {
 	        log(ERR_MEMORY_ALLOCATE, "drop_file", "tmpfilename", strerror(errno));
@@ -921,37 +1014,37 @@ drop_file(GMimeObject *object, struct config *cfg, struct mailzip *mz)
 	}
 	free(tmpname);
 
-        mk_new_filename(newfilename, tmpfilename, mz->zipdir, count);
+        if (ismode_delete) {
+            strcpy(newfilename, tmpfilename);
+        } else {
+            mk_new_filename(newfilename, tmpfilename, mz->zipdir, count);
+        } 
 
         if (push_attnamelist(&(mz->namelist), newfilename) != 0) {
 #ifndef GMIME24
             g_object_unref (att);
 #endif
-	    free(tmpfilename);
+            free(tmpfilename);
             return -1;
         }
 
-	free(tmpfilename);
+        free(tmpfilename);
 
-	snprintf(tmppath, PATH_MAX, FILE_PATH, mz->zipdir, newfilename);
-	DEBUGLOG("attach_file_name :%s\n", tmppath);
-
-	wfp = fopen(tmppath, "w");
-
-	if (wfp == NULL) {
-	    log(ERR_FILE_OPEN, "drop_file", tmppath);
+        snprintf(tmppath, PATH_MAX, FILE_PATH, mz->zipdir, newfilename);
+        DEBUGLOG("attach_file_name :%s\n", tmppath);
 
+        if (GMIME_IS_MESSAGE_PART(att)) {
+            wfp = fopen(tmppath, "w");
+            if (wfp == NULL) {
+                log(ERR_FILE_OPEN, "drop_file", tmppath);
 #ifndef GMIME24
-	    g_object_unref (att);
+                g_object_unref (att);
 #endif
+                return -1;
+            }
+            w_stream = g_mime_stream_file_new (wfp);
 
-	    return -1;
-	}
-
-	w_stream = g_mime_stream_file_new (wfp);
-
-	if (GMIME_IS_MESSAGE_PART(att)) {
-	    message = g_mime_message_part_get_message((GMimeMessagePart *)att);
+            message = g_mime_message_part_get_message((GMimeMessagePart *)att);
 
 #ifndef GMIME26
             ret = g_mime_message_write_to_stream (message, w_stream);
@@ -964,8 +1057,18 @@ drop_file(GMimeObject *object, struct config *cfg, struct mailzip *mz)
 #endif
             g_object_unref (w_stream);
 
-	} else {
-	    wrapper = g_mime_part_get_content_object((GMimePart *)att);
+        } else if (!ismode_delete) {
+            wfp = fopen(tmppath, "w");
+            if (wfp == NULL) {
+                log(ERR_FILE_OPEN, "drop_file", tmppath);
+#ifndef GMIME24
+                g_object_unref (att);
+#endif
+                return -1;
+            }
+            w_stream = g_mime_stream_file_new (wfp);
+
+            wrapper = g_mime_part_get_content_object((GMimePart *)att);
 	    if (wrapper == NULL) {
 	        log(ERR_GMIME, "drop_file", "g_mime_part_get_content_object");
 
@@ -1009,6 +1112,54 @@ drop_file(GMimeObject *object, struct config *cfg, struct mailzip *mz)
     return 0;
 }
 
+/*
+ * mk_deletelistfile
+ *
+ * Function
+ *      Creat a temporary file to save namelist data in.
+ *
+ * Argument
+ *      struct config  *cfg	config structure
+ *      char *tmpdir		temporary directory name
+ *
+ * Return value
+ *      0       Normal end.
+ *      -1      Abormal end.
+ */
+int
+mk_deletelistfile(struct config *cfg, struct mailzip *mz)
+{
+    FILE *fp;
+    int i, retw;
+    
+    /* open a deletelistfile named by mz->encfilepath. */
+    fp = fopen(mz->encfilepath, "w");
+    if (fp == NULL) {
+        log(ERR_FILE_OPEN, "mk_deletelistfile", mz->encfilepath);
+        return -1;
+    }
+    
+    for (i = 0; (mz->namelist + i)->attach_name != NULL; i++) {
+        retw = fwrite((mz->namelist + i)->attach_name, 
+                      (mz->namelist + i)->attach_name_len, 1, fp);
+        if(retw < 0) {
+            log(ERR_FILE_WRITE, "mk_deletelistfile");
+            fclose(fp);
+            return -1;
+        }
+        retw = fprintf(fp, "\r\n");
+        if(retw < 0) {
+            log(ERR_FILE_WRITE, "mk_deletelistfile");
+            fclose(fp);
+            return -1;
+        }
+
+    }
+
+    fclose(fp);
+
+    return (0);
+}
 #define CDCOMMAND	"cd"
 #define QOPTION		"-q"
 #define ANDCOMMAND	"&&"
@@ -1155,23 +1306,27 @@ mk_encpath(struct config *cfg, struct mailzip *mz)
     time_t now;
     char *filename, *path;
     char *tmpdir = mz->zipdir;
+    char *cfg_attachname = NULL;
+
+    /* Switch cfg_attachname according to the run modes */
+    cfg_attachname = ismode_delete ? cfg->cf_deletelistname : cfg->cf_zipfilename;
 
     /* get now time */
     time(&now);
     ti = localtime(&now);
 
     /* allocate date string */
-    strdatenum = strlen(cfg->cf_zipfilename) + STRTIMENUM + 1;
+    strdatenum = strlen(cfg_attachname) + STRTIMENUM + 1;
     filename = malloc(strdatenum);
     if (filename == NULL) {
 	log(ERR_MEMORY_ALLOCATE, "add_enclist", "rcptinfo", strerror(errno));
 	return -1;
     }
 
-    strftime(filename, strdatenum - 1, cfg->cf_zipfilename, ti);
+    strftime(filename, strdatenum - 1, cfg_attachname, ti);
     mz->attachfilename = filename;
 
-    /* create zip file path */
+    /* create zip (delete-report) file path in Encryption (Delete) mode */
     path = malloc(strlen(tmpdir) + strdatenum);
     if (path == NULL) {
 	log(ERR_MEMORY_ALLOCATE, "add_enclist", "rcptinfo", strerror(errno));
