@@ -93,10 +93,12 @@ struct name_list {
 };
 
 int parse_mail(struct mailinfo *, struct config *, struct mailzip *);
+extern int delete_attachments_mail(SMFICTX *, struct mailinfo *, struct config *, char *, struct rcptinfo *);
 extern int zip_convert_mail(SMFICTX *, struct mailinfo *, struct config *, char *, struct rcptinfo *, struct rcptinfo *);
 void g_message_callback(GMimeObject *, gpointer);
 int mk_new_filename(char *, const char *, char *, int);
 int remove_file_with_dir_recursive(char *);
+int mk_deletelistfile(struct config *, struct mailzip *);
 int convert_zip(struct config *, struct mailzip *, struct rcptinfo **);
 int check_ascii(char *);
 int mk_encpath(struct config *, struct mailzip *);
